% Demo_Retrieval_CorelWang code to perform an example code of image
% retrieval on Corel Wang database based on GMMs matching.
%  For  theoretical and technical details, please refer to the following paper:
%
% Peihua Li,  Qilong Wang, Lei Zhang. A Novel Earth Mover's Distance Methodology for Image Matching with 
% Gaussian Mixture Models. IEEE Int. Conf. on Computer Vision (ICCV), 2013.
%
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Qilong Wang  (Email:  wangqilong.415@163.com), 
%                                               Peihua  Li (Email: peihuali at dlut dot edu dot cn) 
%                                               Lei zhang (Email: cslzhang  at comp dot polyu dot edu dot hk)
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

%%
%clear; clc;
curr_dir = pwd;
addpath(fullfile(curr_dir, 'GenerateFeatures'));
addpath( fullfile(curr_dir, 'BuildGMMs'));
addpath( fullfile(curr_dir, 'Common'));

%% Caculate image descriptors. In this demo program, L2ECM descriptors (Li & Wang, ECCV 2012)) are used
% Peihua Li, Qilong Wang: Local Log-Euclidean Covariance Matrix (L2ECM) for Image Representation and Its Applications. ECCV (3) 2012: 469-482
fprintf('------------------------------------------------------------------------------------------\n');
fprintf('Extracting dense descriptors: \n');
start_time = round(clock);

image_dataset =   'WANG';
feature_type    =  'L2ECM'; %  'L2ECM'  

base_image_dir  = ['Images\'  image_dataset];
base_feature_dir =['Data_descriptor\' image_dataset];
base_model_dir  = ['Data_model\' image_dataset];

maxImageSize = 1000;
patch_size = 16;
grid_spacing = 4;

categories_dir0 = dir(base_image_dir);
categories_dir = struct([]);
for i=1:length(categories_dir0)
    if strcmp(categories_dir0(i).name, '.') ~=1 &&   strcmp(categories_dir0(i).name, '..')~=1 && categories_dir0(i).isdir ~=0
        categories_dir = [categories_dir; categories_dir0(i)];
    end
end

% if matlabpool('size') == 0
%     n = feature('numCores');
%     matlabpool('local', n);
% end

canSkip = 1;
file_suffix = '.jpg';
for i=1: length(categories_dir)
    image_dir = fullfile(base_image_dir, sprintf('%s', categories_dir(i).name));
    data_dir   =  fullfile(base_feature_dir,   sprintf('%s', categories_dir(i).name));
  
    CalculateDescriptors(feature_type,   patch_size,  grid_spacing,     maxImageSize,   image_dir,     data_dir,  file_suffix,  canSkip);
end

end_time = round(clock);
time_elapsed = end_time - start_time;
fprintf('Time elapsed is : %d hours %d minutes %d seconds\n',  time_elapsed(4), time_elapsed(5), time_elapsed(6));


%% Compute image models by Gaussian Mixture Models (GMMs)
fprintf('------------------------------------------------------------------------------------------\n');
fprintf('Building image models by Gaussian Mixture Models (GMMs): \n');
start_time = round(clock);

init_K = 12;
final_K = 10;
gmm_sampling_step = 1; 
file_suffix = '.mat';
for i=1: length(categories_dir)
    data_dir     =  fullfile(base_feature_dir,   sprintf('%s', categories_dir(i).name));
    model_dir   =  fullfile(base_model_dir,   sprintf('%s', categories_dir(i).name));
  
    ComputeGMMs(data_dir, model_dir,  file_suffix, gmm_sampling_step, init_K, final_K,  canSkip);
end
end_time = round(clock);
time_elapsed = end_time - start_time;
fprintf('Time elapsed is : %d hours %d minutes %d seconds\n',  time_elapsed(4), time_elapsed(5), time_elapsed(6));


%% Image Retrieval with SR-EMD
fprintf('------------------------------------------------------------------------------------------\n');
fprintf('Performing image retrieval \n');
start_time = round(clock);

SR_EMD_para = struct();
SR_EMD_para.DistanceType = 'EMD-M';  %  'EMD-M'   'EMD-theta'
SR_EMD_para.theta = 0.5;  %default theta
SR_EMD_para.M = eye(16); %M should be symmetric positive definite (SPD) matrix and is the identity matrix in default
[U S V]=svd(SR_EMD_para.M);
SR_EMD_para.M_one_half = U * diag(sqrt(diag(S))) * V';  %the square root matrix of SR_EMD_para.M, i.e. SR_EMD_para.M = SR_EMD_para.M_one_half * SR_EMD_para.M_one_half, which is also a SPD matrix
SR_EMD_para.k = 0;  % used in gmmknnclasify.m

iter_max = 50;
acc = zeros(iter_max, 1);

for  iter = 1: iter_max;
    
    % Select in random the training files
    training_files_all = cell(length(categories_dir), 1);
    test_files_all      = cell(length(categories_dir), 1);
    
    rp_dir = fullfile('Common\WANG\rplist', sprintf('rp%s.mat', num2str(iter)));
    order_dir = fullfile('Common\WANG\rplist\order.mat');
    load(rp_dir);testid = rp; load(order_dir);
    for i=1:length(categories_dir)
        model_dir   =  fullfile(base_model_dir,   sprintf('%s', categories_dir(i).name));
        [training_files test_files] = training_test_images(model_dir, testid(order(i)));
        training_files_all{i} = training_files;
        test_files_all{i}      = test_files;
    end

    % Load models of training images and obtain their labels
    training_models = cell(length(categories_dir), 1);
    for i=1:length(categories_dir)  
        training_models{i}= load_models(training_files_all{i});
    end
    training_labels_all = [];
    for i=1:length(categories_dir)
        training_labels = zeros(length(training_models{i}), 1);
        training_labels(:) = i;
        training_labels_all = [training_labels_all; training_labels];
    end
    training_models_all = [training_models{:}];
    training_models_all = training_models_all(:);
    SR_EMD_para.k = size(training_labels_all,1);
    % Retrieval using SR-EMD
    tempK = 1:SR_EMD_para.k;  
   for i=1:length(categories_dir)
        test_models = load_models(test_files_all{i});
        [retrievalID BestK] = gmmknnclassify(test_models,  training_models_all, training_labels_all, SR_EMD_para); 
        re_label = training_labels_all(retrievalID);
        Ri_ID_idx = double(re_label == i);
        Ri_ID = cumsum(Ri_ID_idx);
        Ri_ID(find(re_label ~=i)) = 0;
        precision = Ri_ID ./ tempK';
        acc(iter,i) = sum(precision) / length(training_models{i});
    end
     fprintf('Round %d MAP on Corel Wang dataset is %f  . \n', iter, mean(acc(iter, :)));
end

 fprintf('Average MAP result %d rounds of  on %s %f  .\n',iter_max, image_dataset,  mean(mean(acc, 2)));
 
 if matlabpool('size') > 0
   matlabpool close;
 end

end_time = round(clock);
time_elapsed = end_time - start_time;
fprintf('Time elapsed is : %d hours %d minutes %d seconds\n',  time_elapsed(4), time_elapsed(5), time_elapsed(6));


