function [feature_cov,feature_cov_mask, grid_x, grid_y] = dense_cov_integral(I_features, patch_size, grid_spacing)
% dense_cov_integral(I_orientation, patch_size, grid_spacing) computes
% dense covariance matrices
%
% Input:
% I_features: raw feature image
% patch_size:the size of a patch on which to compute the descriptor
% grid_spacing: The sampling spacing of dense descriptors
%
% Output: 
%feature_cov: dense  features--matrixlogarithm of covariance features
%grid_x, grid_y: the cooridnates of dense features
%
%
% Please cite the following paper if you use the code:
%
%Peihua Li,  Qilong Wang: Local Log-Euclidean Covariance Matrix (L2ECM) for Image 
%Representation and Its Applications, (ECCV) 2012.
%
% For questions,  please conact:  Qilong Wang  (Email:  wangqilong.415@163.com), 
%                                               Peihua  Li (Email: peihuali at dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

sample_step = grid_spacing;
grid_spacing = 1;

[hgt wid num_angles] = size(I_features);
n = num_angles;
pixels_num = patch_size * patch_size;

% compute the integral images
inte_features = integralfeatures(I_features);

grid_x = patch_size/2:grid_spacing:wid-patch_size/2;
grid_y = patch_size/2:grid_spacing:hgt-patch_size/2;
[sx1 sx] = size(grid_x);
[sy1 sy] = size(grid_y);


[Hgt Wid layer] = size(inte_features);
window_sum = zeros(sy, sx, layer);

for i = 1 : layer
     window_sum(:,:,i) = inte_features(patch_size+1 : end, patch_size+1 : end, i) + inte_features(1 : end-patch_size, 1 : end-patch_size, i) ...
                                 - inte_features(patch_size+1 : end, 1 : end-patch_size, i) - inte_features(1 : end-patch_size, patch_size+1 : end, i);
end

cov_vector1 = zeros(sy, sx, n, n);
layer_idx = n;
for i = 1 : n
    for j = i : n
        layer_idx =layer_idx + 1;
        cov_vector1(:,:,i,j) = window_sum(:,:,layer_idx) / (pixels_num-1) - window_sum(:,:,i) .* window_sum(:,:,j)  / ((pixels_num-1)* pixels_num);
        cov_vector1(:,:,j,i) = cov_vector1(:,:,i,j);
    end
end

data_cov = zeros(num_angles, num_angles);
U=zeros(num_angles,num_angles);
S=zeros(num_angles,num_angles);
V=zeros(num_angles,num_angles);
diag_S=zeros(num_angles,1);


[h_s w_s d_s]= size(cov_vector1);
sample_grid_y = 1:sample_step:h_s;
sample_grid_x = 1:sample_step:w_s;
feature_cov = zeros(length(sample_grid_y), length(sample_grid_x), d_s);
feature_cov_mask = zeros(length(sample_grid_y), length(sample_grid_x), 'uint8');   
for y_idx= 1:length(sample_grid_y)
    for x_idx = 1:length(sample_grid_x)
        data_cov = reshape(cov_vector1(sample_grid_y(y_idx),sample_grid_x(x_idx),:,:), n, n);

        [U S V]=svd(data_cov);
        diag_S=diag(S);
        idx = find(diag_S>num_angles*eps(diag_S(1,1)));
        feature_cov_mask(y_idx,x_idx) = length(idx);
        data_cov = U*diag(log(diag_S))*V';
        
        feature_cov(y_idx, x_idx,:)= data_cov(:);
    end
end

grid_y = grid_y(1:sample_step:h_s);
grid_x = grid_x(1:sample_step:w_s);

end



